-- Supporting code for www.adellera.it/blog
--
-- (c) Alberto Dell'Era, October 2009
-- Tested in 9.2.0.8, 11.1.0.7.

set echo on
set lines 150
set pages 9999
set serveroutput on size 1000000
set trimspool on 

drop table t;

variable db_version varchar2(30)
declare
  l_dummy varchar2(100);
begin
  dbms_utility.db_version (:db_version, l_dummy);
end;
/

col db_version new_value db_version
select replace ( rtrim(:db_version,'.0'), '.', '_') as db_version from dual;

spool density_post_freq_&db_version..lst

-- the test table with an exponential value distribution
create table t (value int);

-- uncomment the next row to see what happens with bkt(least_popular_value) = 1
--insert into t(value) select 1   from dual connect by level <= 1;
--insert into t(value) select 2   from dual connect by level <= 2;
--insert into t(value) select 4   from dual connect by level <= 4;
insert into t(value) select 8   from dual connect by level <= 8;
insert into t(value) select 16  from dual connect by level <= 16;
insert into t(value) select 64  from dual connect by level <= 64;
insert into t(value) select 128 from dual connect by level <= 128;
commit;

--alter session set "_optimizer_enable_density_improvements"=false;

select value, count(*)
  from t
 group by value
 order by value;

-- gather stats and histogram
exec dbms_stats.gather_table_stats (user, 'T', method_opt=>'for all columns size 254', estimate_percent=>null);

-- a view to format dba_histograms for our example table
create or replace view formatted_hist as
with hist1 as (
  select endpoint_number ep, endpoint_value value
    from user_histograms 
   where table_name  = 'T'
     and column_name = 'VALUE'
), hist2 as (
  select ep, value, 
         lag (ep) over (order by ep) prev_ep,
         max (ep) over ()            max_ep
    from hist1
)
select value, ep, ep - nvl(prev_ep,0) as bkt,
       decode (ep - nvl (prev_ep, 0), 0, 0, 1, 0, 1) as popularity
 from hist2
order by ep;

-- print (formatted) histograms
select value, ep, bkt from formatted_hist;
       
-- print density and related figures
select c.density * t.num_rows, c.density, histogram
  from user_tab_columns c, user_tables t
 where c.table_name = 'T' and c.column_name = 'VALUE'
   and c.table_name = t.table_name;

-- fire the CBO on the equality filter predicate
alter session set tracefile_identifier=density_post_freq_&db_version.;
show parameter user_dump_dest;

alter session set events '10053 trace name context forever, level 1';
select * from t not_existent where value = 64.5; 
alter session set events '10053 trace name context off';

alter session set "_fix_control"='5483301:off';

alter session set events '10053 trace name context forever, level 1'; 
select * from t not_existent_off where value = 64.5; 
alter session set events '10053 trace name context off';
  
alter session set "_fix_control"='5483301:on';

spool off

